
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.3601;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.3601"
  },
  "configure_requires" => {
    "Module::Build" => "0.3601"
  },
  "dist_abstract" => "Query engine and query language for trees in PML format",
  "dist_author" => [
    "Petr Pajas <pajas\@ufal.mff.cuni.cz>",
    "Jan \x{160}t\x{11b}p\x{e1}nek <stepanek\@ufal.mff.cuni.cz>",
    "Michal Sedl\x{e1}k <sedlak\@ufal.mff.cuni.cz>",
    "Maty\x{e1}\x{161} Kopp <matyas.kopp\@gmail.com>"
  ],
  "dist_name" => "PMLTQ",
  "dist_version" => "2.0.2",
  "license" => "perl",
  "module_name" => "PMLTQ",
  "recommends" => {
    "DBD::Pg" => "v2.19.3",
    "Net::HTTPServer" => "v1.1.1"
  },
  "recursive_test_files" => 1,
  "requires" => {
    "Benchmark" => "1.12",
    "DBI" => 0,
    "File::ShareDir" => "1.102",
    "File::Slurp" => 0,
    "Getopt::Long" => "2.48",
    "Graph" => "0.96",
    "Graph::ChuLiuEdmonds" => "0.06",
    "IO::Scalar" => "2.110",
    "List::MoreUtils" => 0,
    "List::Util" => "1.45",
    "Mojo::Base::XS" => 0,
    "Pod::Usage" => "1.67",
    "Text::Balanced" => 0,
    "Time::HiRes" => "1.97",
    "Treex::PML" => "2.18",
    "YAML" => "0.88",
    "YAML::Tiny" => 0
  },
  "share_dir" => {
    "dist" => "share"
  },
  "test_requires" => {
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::More" => 0,
    "Test::Most" => 0
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.3601",
  "Test::More" => 0,
  "Test::Most" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

if ( $^O =~ /^(aix|android|bitrig|bsdos|cygwin|darwin|dgux|dragonfly|dynixptx|freebsd|gnukfreebsd|hpux|interix|irix|linux|machten|midnightbsd|mirbsd|netbsd|next|nto|openbsd|qnx|sco|solaris)$/i ) {
	$module_build_args{requires}{'Sys::SigAction'} = '0.21';
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
