@import "main.css";

body, th, td { 
	color: #0E0E61; /* 1-1 */
}

h1.title, h2.title, h3.title, h4.title {
	color: #020293; /* 1-4 */
}

h5.title, h6.title {
	color: #121244; /* 1-2 */
}

code, .programlisting, .literallayout, .literal, .synopsis {
	color: #050530;
}

.programlisting, .literallayout, .synopsis {
	background: #d5dee6;
        padding: 4pt 4pt 4pt 4pt;
}

.programlisting > p, .literallayout > p {
  padding: 0 0 0 0;
  margin: 0 0 0 0;
}

/* navigace */

div.navfooter a, div.navheader a {
	color: #610E61; /* 2-1 */
}

div.navfooter a:hover, div.navheader a:hover {
	background: #FFC8FF; /* 2-3 */
}

div.navheader th
{
	color: #1A1A61; /* 1-2 */
}

/* odkazy */

a {
	color: #0E6161; /* 3-1 *//* vyjimka-blizsi */
}

a:hover {
	background: #C8FFE8; /* 3-3 */
}

div.added    {  background-color: #ddffdd; } 
div.deleted  { text-decoration: line-through;
               background-color: #FF9F9F; }
div.changed  { 
               background-color: #ffffdd;
             }
div.off      {  }

span.added   { background-color: #ffffcc; }
span.deleted { text-decoration: line-through;
               background-color: #FF7F7F; }
span.changed { background-color: #ccffcc; }
span.off     {  }

dt { font-weight: bold }

.framed, .note , .warning, .tip{ 
   background-color: #ffffcc;
   border: dashed 1pt;
   padding: 6pt 12pt 6pt 12pt;
   margin: 0 0 0 0;
}

.framed, .note, .tip  { 
   background-color: #ffffcc;
}

.warning {
   background-color: #ffdede;
}

.framed > h3, .note > h3, .warning > h3, .tip > h3 {
  margin: 0 0 0 0;
}

.guibutton img {
  vertical-align: -30%;
  color: black;
  background: #f0f0f0;
  border: 1px solid;
  border-color: #d0d0d0 #505050  #606060 #c0c0c0;
  font-family: monospace;
  font-size: 80%;
  padding: 3px;
}

.inlinemediaobject img {
  vertical-align: -30%;
}

.keysym { 
color: black;
background: #f0f0f0;
border: 1px solid;
border-color: #d0d0d0 #505050  #606060 #c0c0c0;
padding: 0px 2px 0px 2px;
font-family: monospace;
font-size: 100%;
padding: 2px 3px 2px 3px;
}

