body {
	margin: 10pt 10pt;
	text-align: justify;
}

li > p, td > p {
	margin: 0.3em;
	padding: 0.3em;
}

body, th, td {
    max-width: 70em;
    background-color: #FFFFFF;
}

/* nadpisy */

h1, h2, h3, h4, h5, h6 {
	text-align: left;
}

h1.title, h2.title, h3.title, h4.title, h5.title, h6.title {
/*	text-transform: uppercase; */
	margin-top: 16pt;
}

h1.title img {
  margin: -16pt 0px 0px -10pt;
}



h4.title, h5.title, h6.title {
	margin-top: 16pt;
}


h3.title {
/*	border-bottom: thin solid; */
}	

h4.title {
/*	border-bottom: thin solid; */
	margin-right: 50%;
}	

/* fonty */

body, td, th {
	font-family: Trebuchet MS, Arial, Helvetica, sans-serif; 
	font-size: 10pt;
}

code, .programlisting, .literallayout {
	font-family: Courier, monospace;
}

/* obrazky */

div.figure {
	text-align: center;
	/*border-width: thin;
	border-style: solid;*/
	margin: 0.5em;
	padding: 0.5em;
}

div.figure img {
	display: block;
	margin-left: auto;
	margin-right: auto;
}

div.figure p {
	text-align: center;
}

div.figure + div.figure {
	margin-top: 1.66em;
}

/* tabulky */

div.table + div.table {
	margin-top: 1.66em;
}

div.table p {
	text-align: left; 
}

div.table table {
	display: table;
	margin-left: auto;
	margin-right: auto;
}

table {
	border-collapse: collapse;
	border-style: none;
}

td, th { 
	border-style: solid;
	border: inset 1pt; 
	_border: inset 2pt;
	padding: 0.2em;
        padding-top: 0em;
        padding-bottom: 0em;
}

table[border="0"] td, table[border="0"] th {
	border-style: none;
}

/* navigacni zahlavi a zapati */

div.navfooter td, div.navheader td, div.navheader th {
	border: inset 0pt;
}

*
div.navheader {
	margin-bottom: -1.17em;
}

div.navfooter td {
	color: #BBBBBB;
}

/* odkazy */

a {
	font-weight: bold; 
	text-decoration: none
}

/* index */

div.index a { 
  font-weight: normal; 
  text-decoration: none
}

div.index em { 
  font-weight: bold; 
  font-style: normal; 
}

div.index em a { 
  font-weight: bold; 
  text-decoration: none
}

.math {
  font-family: "Times New Roman", serif; 
  /* font-style: italic;  */
}

.right {
    border: none;
    float: right;
    margin: 0.5em 0 0.5em 1em; 
    clear: right;
}

.top-links {
    text-align: center;
    background: #eeeeee;
    margin: 0pt 0pt 5pt 0pt;
    padding: 3pt 5pt 3pt 5pt;
}
.top-links a {
    font-weight: normal;
    font-size: 90%;
    margin: 10pt 10pt 15pt 10pt;
    color: black;
}

.web-section {
    margin:  0pt 0pt 0pt 10pt;
}

